@@ -21,9 +21,24 @@ mod geos;
 mod overlay;
 pub mod register;
 mod st_area;
+mod st_boundary;
 mod st_buffer;
 mod st_centroid;
 mod st_convexhull;
 mod st_dwithin;
+mod st_isring;
+mod st_issimple;
+mod st_isvalid;
+mod st_isvalidreason;
 mod st_length;
+mod st_makevalid;
+mod st_minimumclearance;
+mod st_minimumclearance_line;
 mod st_perimeter;
+mod st_polygonize;
+mod st_polygonize_agg;
+mod st_simplify;
+mod st_simplifypreservetopology;
+mod st_snap;
+mod st_unaryunion;
+pub mod wkb_to_geos;
