@@ -14,7 +14,6 @@
 // KIND, either express or implied.  See the License for the
 // specific language governing permissions and limitations
 // under the License.
-use std::{env, path::PathBuf};
 
 fn main() {
     println!("cargo:rerun-if-changed=src/tg/tg.c");
@@ -25,17 +24,4 @@ fn main() {
         .flag_if_supported("/std:c11")
         .flag_if_supported("/experimental:c11atomics")
         .compile("tg");
-
-    let bindings = bindgen::Builder::default()
-        .header("src/tg/tg.h")
-        .generate_comments(false)
-        .parse_callbacks(Box::new(bindgen::CargoCallbacks::new()))
-        .generate()
-        .expect("Unable to generate bindings");
-
-    // Write the bindings to the $OUT_DIR/bindings.rs file.
-    let out_path = PathBuf::from(env::var("OUT_DIR").unwrap());
-    bindings
-        .write_to_file(out_path.join("bindings.rs"))
-        .expect("Couldn't write bindings!");
 }
