@@ -16,7 +16,7 @@
 # under the License.
 
 from pathlib import Path
-from typing import TYPE_CHECKING, Union, Optional, Any, Iterable
+from typing import TYPE_CHECKING, Union, Optional, Any, Iterable, Literal
 
 from sedonadb.utility import sedona  # noqa: F401
 
@@ -58,6 +58,11 @@ class DataFrame:
         """
         return self._impl.schema()
 
+    @property
+    def columns(self) -> list[str]:
+        """Return a list of column names"""
+        return self._impl.columns()
+
     def head(self, n: int = 5) -> "DataFrame":
         """Limit result to the first n rows
 
@@ -295,13 +300,15 @@ class DataFrame:
         partition_by: Optional[Union[str, Iterable[str]]] = None,
         sort_by: Optional[Union[str, Iterable[str]]] = None,
         single_file_output: Optional[bool] = None,
+        geoparquet_version: Literal["1.0", "1.1"] = "1.0",
+        overwrite_bbox_columns: bool = False,
     ):
         """Write this DataFrame to one or more (Geo)Parquet files
 
         For input that contains geometry columns, GeoParquet metadata is written
         such that suitable readers can recreate Geometry/Geography types when
-        reading the output.
-
+        reading the output and potentially read fewer row groups when only a
+        subset of the file is needed for a given query.
 
         Args:
             path: A filename or directory to which parquet file(s) should be written.
@@ -313,6 +320,21 @@ class DataFrame:
                 file vs. writing one file per partition to a directory. By default,
                 a single file is written if `partition_by` is unspecified and
                 `path` ends with `.parquet`.
+            geoparquet_version: GeoParquet metadata version to write if output contains
+                one or more geometry columns. The default (1.0) is the most widely
+                supported and will result in geometry columns being recognized in many
+                readers; however, only includes statistics at the file level.
+
+                Use GeoParquet 1.1 to compute an additional bounding box column
+                for every geometry column in the output: some readers can use these columns
+                to prune row groups when files contain an effective spatial ordering.
+                The extra columns will appear just before their geometry column and
+                will be named "[geom_col_name]_bbox" for all geometry columns except
+                "geometry", whose bounding box column name is just "bbox".
+            overwrite_bbox_columns: Use `True` to overwrite any bounding box columns
+                that already exist in the input. This is useful in a read -> modify
+                -> write scenario to ensure these columns are up-to-date. If `False`
+                (the default), an error will be raised if a bbox column already exists.
 
         Examples:
 
@@ -344,7 +366,13 @@ class DataFrame:
             sort_by = []
 
         self._impl.to_parquet(
-            self._ctx, str(path), partition_by, sort_by, single_file_output
+            self._ctx,
+            str(path),
+            partition_by,
+            sort_by,
+            single_file_output,
+            geoparquet_version,
+            overwrite_bbox_columns,
         )
 
     def show(
@@ -413,7 +441,7 @@ class DataFrame:
             │      utf8     ┆               utf8              │
             ╞═══════════════╪═════════════════════════════════╡
             │ logical_plan  ┆ Projection: Int64(1) AS one     │
-            │               ┆   EmptyRelation                 │
+            │               ┆   EmptyRelation: rows=1         │
             ├╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┼╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌╌┤
             │ physical_plan ┆ ProjectionExec: expr=[1 as one] │
             │               ┆   PlaceholderRowExec            │
