@@ -41,6 +41,20 @@ pub struct SedonaScalarUDF {
     aliases: Vec<String>,
 }
 
+impl PartialEq for SedonaScalarUDF {
+    fn eq(&self, other: &Self) -> bool {
+        self.name == other.name
+    }
+}
+
+impl Eq for SedonaScalarUDF {}
+
+impl std::hash::Hash for SedonaScalarUDF {
+    fn hash<H: std::hash::Hasher>(&self, state: &mut H) {
+        self.name.hash(state);
+    }
+}
+
 /// User-defined function implementation
 ///
 /// A `SedonaScalarUdf` is comprised of one or more kernels, to which it dispatches
@@ -83,6 +97,16 @@ pub trait SedonaScalarKernel: Debug {
         arg_types: &[SedonaType],
         args: &[ColumnarValue],
     ) -> Result<ColumnarValue>;
+
+    fn invoke_batch_from_args(
+        &self,
+        arg_types: &[SedonaType],
+        args: &[ColumnarValue],
+        _return_type: &SedonaType,
+        _num_rows: usize,
+    ) -> Result<ColumnarValue> {
+        self.invoke_batch(arg_types, args)
+    }
 }
 
 /// Type definition for a Scalar kernel implementation function
@@ -139,19 +163,13 @@ impl SedonaScalarUDF {
         }
     }
 
-    pub fn new_with_aliases(
-        name: &str,
-        kernels: Vec<ScalarKernelRef>,
-        volatility: Volatility,
-        documentation: Option<Documentation>,
-        aliases: Vec<String>,
-    ) -> SedonaScalarUDF {
-        let signature = Signature::user_defined(volatility);
+    /// Add aliases to an existing SedonaScalarUDF
+    pub fn with_aliases(self, aliases: Vec<String>) -> SedonaScalarUDF {
         Self {
-            name: name.to_string(),
-            signature,
-            kernels,
-            documentation,
+            name: self.name,
+            signature: self.signature,
+            kernels: self.kernels,
+            documentation: self.documentation,
             aliases,
         }
     }
@@ -265,8 +283,8 @@ impl ScalarUDFImpl for SedonaScalarUDF {
             })
             .collect::<Vec<_>>();
 
-        let (kernel, _) = self.return_type_impl(&arg_types, &arg_scalars)?;
-        kernel.invoke_batch(&arg_types, &args.args)
+        let (kernel, return_type) = self.return_type_impl(&arg_types, &arg_scalars)?;
+        kernel.invoke_batch_from_args(&arg_types, &args.args, &return_type, args.number_rows)
     }
 
     fn aliases(&self) -> &[String] {
