@@ -20,10 +20,10 @@ mod st_area;
 mod st_centroid;
 mod st_distance;
 mod st_dwithin;
-mod st_intersection_aggr;
+mod st_intersection_agg;
 mod st_intersects;
 mod st_length;
 mod st_line_interpolate_point;
 mod st_perimeter;
-mod st_union_aggr;
+mod st_union_agg;
 pub mod to_geo;
