@@ -16,7 +16,10 @@
 // under the License.
 use serde::{Deserialize, Serialize};
 
-use crate::interval::{Interval, IntervalTrait, WraparoundInterval};
+use crate::{
+    error::SedonaGeometryError,
+    interval::{Interval, IntervalTrait, WraparoundInterval},
+};
 
 /// Bounding Box implementation with wraparound support
 ///
@@ -162,6 +165,25 @@ impl BoundingBox {
             _ => None,
         };
     }
+
+    /// Compute the intersection of this bounding box with another
+    ///
+    /// This method will propagate missingness of Z or M dimensions from the two boxes
+    /// (e.g., Z will be `None` if Z if `self.z().is_none()` OR `other.z().is_none()`).
+    pub fn intersection(&self, other: &Self) -> Result<Self, SedonaGeometryError> {
+        Ok(Self {
+            x: self.x.intersection(&other.x)?,
+            y: self.y.intersection(&other.y)?,
+            z: match (self.z, other.z) {
+                (Some(z), Some(other_z)) => Some(z.intersection(&other_z)?),
+                _ => None,
+            },
+            m: match (self.m, other.m) {
+                (Some(m), Some(other_m)) => Some(m.intersection(&other_m)?),
+                _ => None,
+            },
+        })
+    }
 }
 
 #[cfg(test)]
@@ -358,6 +380,53 @@ mod test {
         assert!(bounding_box.m().is_none());
     }
 
+    #[test]
+    fn bounding_box_intersection() {
+        assert_eq!(
+            BoundingBox::xy((1, 2), (3, 4))
+                .intersection(&BoundingBox::xy((1.5, 2.5), (3.5, 4.5)))
+                .unwrap(),
+            BoundingBox::xy((1.5, 2.0), (3.5, 4.0))
+        );
+
+        // If z and m are present in one input but not the other, we propagate the unknownness
+        // to the intersection
+        assert_eq!(
+            BoundingBox::xyzm(
+                (1, 2),
+                (3, 4),
+                Some(Interval::empty()),
+                Some(Interval::empty())
+            )
+            .intersection(&BoundingBox::xy((1.5, 2.5), (3.5, 4.5)))
+            .unwrap(),
+            BoundingBox::xy((1.5, 2.0), (3.5, 4.0))
+        );
+
+        // If z and m are specified in both, we include the intersection in the output
+        assert_eq!(
+            BoundingBox::xyzm(
+                (1, 2),
+                (3, 4),
+                Some(Interval::empty()),
+                Some(Interval::empty())
+            )
+            .intersection(&BoundingBox::xyzm(
+                (1.5, 2.5),
+                (3.5, 4.5),
+                Some(Interval::empty()),
+                Some(Interval::empty())
+            ))
+            .unwrap(),
+            BoundingBox::xyzm(
+                (1.5, 2.0),
+                (3.5, 4.0),
+                Some(Interval::empty()),
+                Some(Interval::empty())
+            )
+        );
+    }
+
     fn check_serialize_deserialize_roundtrip(bounding_box: BoundingBox) {
         let json_bytes = serde_json::to_vec(&bounding_box).unwrap();
         let bounding_box_roundtrip: BoundingBox = serde_json::from_slice(&json_bytes).unwrap();
