@@ -18,23 +18,61 @@ use std::{collections::HashMap, sync::Arc};
 
 use arrow_schema::SchemaRef;
 use datafusion::datasource::{
-    file_format::parquet::fetch_parquet_metadata,
     listing::PartitionedFile,
     physical_plan::{parquet::ParquetAccessPlan, FileMeta, FileOpenFuture, FileOpener},
 };
 use datafusion_common::Result;
+use datafusion_datasource_parquet::metadata::DFParquetMetadata;
 use datafusion_physical_expr::PhysicalExpr;
+use datafusion_physical_plan::metrics::{Count, ExecutionPlanMetricsSet, MetricBuilder};
 use object_store::ObjectStore;
 use parquet::file::{
     metadata::{ParquetMetaData, RowGroupMetaData},
     statistics::Statistics,
 };
-use sedona_expr::{spatial_filter::SpatialFilter, statistics::GeoStatistics};
+use sedona_expr::{
+    spatial_filter::{SpatialFilter, TableGeoStatistics},
+    statistics::GeoStatistics,
+};
 use sedona_geometry::bounding_box::BoundingBox;
 use sedona_schema::{datatypes::SedonaType, matchers::ArgMatcher};
 
 use crate::metadata::GeoParquetMetadata;
 
+#[derive(Clone)]
+struct GeoParquetFileOpenerMetrics {
+    /// How many file ranges are pruned by [`SpatialFilter`]
+    ///
+    /// Note on "file range": an opener may read only part of a file rather than the
+    /// entire file; that portion is referred to as the "file range". See [`PartitionedFile`]
+    /// for details.
+    files_ranges_spatial_pruned: Count,
+    /// How many file ranges are matched by [`SpatialFilter`]
+    files_ranges_spatial_matched: Count,
+    /// How many row groups are pruned by [`SpatialFilter`]
+    ///
+    /// Note: row groups skipped during the file-level pruning step are not counted
+    /// again here.
+    row_groups_spatial_pruned: Count,
+    /// How many row groups are matched by [`SpatialFilter`]
+    row_groups_spatial_matched: Count,
+}
+
+impl GeoParquetFileOpenerMetrics {
+    fn new(execution_plan_global_metrics: &ExecutionPlanMetricsSet) -> Self {
+        Self {
+            files_ranges_spatial_pruned: MetricBuilder::new(execution_plan_global_metrics)
+                .global_counter("files_ranges_spatial_pruned"),
+            files_ranges_spatial_matched: MetricBuilder::new(execution_plan_global_metrics)
+                .global_counter("files_ranges_spatial_matched"),
+            row_groups_spatial_pruned: MetricBuilder::new(execution_plan_global_metrics)
+                .global_counter("row_groups_spatial_pruned"),
+            row_groups_spatial_matched: MetricBuilder::new(execution_plan_global_metrics)
+                .global_counter("row_groups_spatial_matched"),
+        }
+    }
+}
+
 /// Geo-aware [FileOpener] implementing file and row group pruning
 ///
 /// Pruning happens (for Parquet) in the [FileOpener], so we implement
@@ -47,6 +85,7 @@ pub struct GeoParquetFileOpener {
     predicate: Arc<dyn PhysicalExpr>,
     file_schema: SchemaRef,
     enable_pruning: bool,
+    metrics: GeoParquetFileOpenerMetrics,
 }
 
 impl GeoParquetFileOpener {
@@ -58,6 +97,7 @@ impl GeoParquetFileOpener {
         predicate: Arc<dyn PhysicalExpr>,
         file_schema: SchemaRef,
         enable_pruning: bool,
+        execution_plan_global_metrics: &ExecutionPlanMetricsSet,
     ) -> Self {
         Self {
             inner,
@@ -66,6 +106,7 @@ impl GeoParquetFileOpener {
             predicate,
             file_schema,
             enable_pruning,
+            metrics: GeoParquetFileOpenerMetrics::new(execution_plan_global_metrics),
         }
     }
 }
@@ -75,13 +116,11 @@ impl FileOpener for GeoParquetFileOpener {
         let self_clone = self.clone();
 
         Ok(Box::pin(async move {
-            let parquet_metadata = fetch_parquet_metadata(
-                &self_clone.object_store,
-                &file_meta.object_meta,
-                self_clone.metadata_size_hint,
-                None,
-            )
-            .await?;
+            let parquet_metadata =
+                DFParquetMetadata::new(&self_clone.object_store, &file_meta.object_meta)
+                    .with_metadata_size_hint(self_clone.metadata_size_hint)
+                    .fetch_metadata()
+                    .await?;
 
             let mut access_plan = ParquetAccessPlan::new_all(parquet_metadata.num_row_groups());
 
@@ -96,6 +135,7 @@ impl FileOpener for GeoParquetFileOpener {
                         &mut access_plan,
                         &spatial_filter,
                         &geoparquet_metadata,
+                        &self_clone.metrics,
                     )?;
 
                     filter_access_plan_using_geoparquet_covering(
@@ -104,6 +144,7 @@ impl FileOpener for GeoParquetFileOpener {
                         &spatial_filter,
                         &geoparquet_metadata,
                         &parquet_metadata,
+                        &self_clone.metrics,
                     )?;
                 }
             }
@@ -135,12 +176,18 @@ fn filter_access_plan_using_geoparquet_file_metadata(
     access_plan: &mut ParquetAccessPlan,
     spatial_filter: &SpatialFilter,
     metadata: &GeoParquetMetadata,
+    metrics: &GeoParquetFileOpenerMetrics,
 ) -> Result<()> {
-    let table_geo_stats = geoparquet_file_geo_stats(file_schema, metadata)?;
-    if !spatial_filter.evaluate(&table_geo_stats) {
+    let column_geo_stats = geoparquet_file_geo_stats(file_schema, metadata)?;
+    let table_geo_stats =
+        TableGeoStatistics::try_from_stats_and_schema(&column_geo_stats, file_schema)?;
+    if !spatial_filter.evaluate(&table_geo_stats)? {
+        metrics.files_ranges_spatial_pruned.add(1);
         for i in access_plan.row_group_indexes() {
             access_plan.skip(i);
         }
+    } else {
+        metrics.files_ranges_spatial_matched.add(1);
     }
 
     Ok(())
@@ -156,6 +203,7 @@ fn filter_access_plan_using_geoparquet_covering(
     spatial_filter: &SpatialFilter,
     metadata: &GeoParquetMetadata,
     parquet_metadata: &ParquetMetaData,
+    metrics: &GeoParquetFileOpenerMetrics,
 ) -> Result<()> {
     let row_group_indices_to_scan = access_plan.row_group_indexes();
 
@@ -171,12 +219,19 @@ fn filter_access_plan_using_geoparquet_covering(
     // Iterate through the row groups
     for i in row_group_indices_to_scan {
         // Generate row group statistics based on the covering statistics
-        let row_group_geo_stats =
+        let row_group_column_geo_stats =
             row_group_covering_geo_stats(parquet_metadata.row_group(i), &covering_specs);
+        let row_group_geo_stats = TableGeoStatistics::try_from_stats_and_schema(
+            &row_group_column_geo_stats,
+            file_schema,
+        )?;
 
         // Evaluate predicate!
-        if !spatial_filter.evaluate(&row_group_geo_stats) {
+        if !spatial_filter.evaluate(&row_group_geo_stats)? {
+            metrics.row_groups_spatial_pruned.add(1);
             access_plan.skip(i);
+        } else {
+            metrics.row_groups_spatial_matched.add(1);
         }
     }
 
