@@ -30,12 +30,12 @@ use wkb::reader::Wkb;
 
 use crate::{
     index::IndexQueryResult,
-    init_once_array::InitOnceArray,
     refine::{
         exec_mode_selector::{get_or_update_execution_mode, ExecModeSelector, SelectOptimalMode},
         IndexQueryResultRefiner,
     },
     spatial_predicate::{RelationPredicate, SpatialPredicate, SpatialRelationType},
+    utils::init_once_array::InitOnceArray,
 };
 
 /// TG-specific optimal mode selector that chooses the best execution mode
