@@ -27,22 +27,21 @@ use futures::stream::StreamExt;
 use futures::{ready, task::Poll};
 use parking_lot::Mutex;
 use sedona_common::sedona_internal_err;
-use sedona_functions::st_analyze_aggr::AnalyzeAccumulator;
+use sedona_functions::st_analyze_agg::AnalyzeAccumulator;
 use sedona_schema::datatypes::WKB_GEOMETRY;
 use std::collections::HashMap;
 use std::ops::Range;
 use std::sync::Arc;
 
+use crate::evaluated_batch::EvaluatedBatch;
 use crate::index::SpatialIndex;
-use crate::once_fut::{OnceAsync, OnceFut};
-use crate::operand_evaluator::{
-    create_operand_evaluator, distance_value_at, EvaluatedGeometryArray, OperandEvaluator,
-};
+use crate::operand_evaluator::{create_operand_evaluator, distance_value_at, OperandEvaluator};
 use crate::spatial_predicate::SpatialPredicate;
-use crate::utils::{
+use crate::utils::join_utils::{
     adjust_indices_by_join_type, apply_join_filter_to_indices, build_batch_from_indices,
     get_final_indices_from_bit_map, need_produce_result_in_final,
 };
+use crate::utils::once_fut::{OnceAsync, OnceFut};
 use arrow::array::RecordBatch;
 use arrow::datatypes::{Schema, SchemaRef};
 use sedona_common::option::SpatialJoinOptions;
@@ -151,7 +150,7 @@ impl SpatialJoinProbeMetrics {
                 .counter("probe_input_batches", partition),
             probe_input_rows: MetricBuilder::new(metrics).counter("probe_input_rows", partition),
             output_batches: MetricBuilder::new(metrics).counter("output_batches", partition),
-            output_rows: MetricBuilder::new(metrics).counter("output_rows", partition),
+            output_rows: MetricBuilder::new(metrics).output_rows(partition),
             join_result_candidates: MetricBuilder::new(metrics)
                 .counter("join_result_candidates", partition),
             join_result_count: MetricBuilder::new(metrics).counter("join_result_count", partition),
@@ -415,8 +414,10 @@ impl SpatialJoinStream {
 
         SpatialJoinBatchIterator::new(SpatialJoinBatchIteratorParams {
             spatial_index: spatial_index.clone(),
-            probe_batch: probe_batch.clone(),
-            geom_array,
+            probe_evaluated_batch: EvaluatedBatch {
+                batch: probe_batch,
+                geom_array,
+            },
             join_metrics: self.join_metrics.clone(),
             max_batch_size: self.target_output_batch_size,
             probe_side_ordered: self.probe_side_ordered,
@@ -456,14 +457,10 @@ struct PartialBuildBatch {
 pub(crate) struct SpatialJoinBatchIterator {
     /// The spatial index reference
     spatial_index: Arc<SpatialIndex>,
-    /// The probe batch being processed
-    probe_batch: RecordBatch,
-    /// The geometry batch result from evaluating the probe batch
-    geom_array: EvaluatedGeometryArray,
+    /// The probe side batch being processed
+    probe_evaluated_batch: EvaluatedBatch,
     /// Current probe row index being processed
     current_probe_idx: usize,
-    /// Current rect index being processed
-    current_rect_idx: usize,
     /// Join metrics for tracking performance
     join_metrics: SpatialJoinProbeMetrics,
     /// Maximum batch size before yielding a result
@@ -485,8 +482,7 @@ pub(crate) struct SpatialJoinBatchIterator {
 /// Parameters for creating a SpatialJoinBatchIterator
 pub(crate) struct SpatialJoinBatchIteratorParams {
     pub spatial_index: Arc<SpatialIndex>,
-    pub probe_batch: RecordBatch,
-    pub geom_array: EvaluatedGeometryArray,
+    pub probe_evaluated_batch: EvaluatedBatch,
     pub join_metrics: SpatialJoinProbeMetrics,
     pub max_batch_size: usize,
     pub probe_side_ordered: bool,
@@ -498,10 +494,8 @@ impl SpatialJoinBatchIterator {
     pub(crate) fn new(params: SpatialJoinBatchIteratorParams) -> Result<Self> {
         Ok(Self {
             spatial_index: params.spatial_index,
-            probe_batch: params.probe_batch,
-            geom_array: params.geom_array,
+            probe_evaluated_batch: params.probe_evaluated_batch,
             current_probe_idx: 0,
-            current_rect_idx: 0,
             join_metrics: params.join_metrics,
             max_batch_size: params.max_batch_size,
             probe_side_ordered: params.probe_side_ordered,
@@ -524,9 +518,10 @@ impl SpatialJoinBatchIterator {
         // Process probe rows incrementally until we have enough results or finish
         let initial_size = self.build_batch_positions.len();
 
-        let wkbs = self.geom_array.wkbs();
-        let rects = &self.geom_array.rects;
-        let distance = &self.geom_array.distance;
+        let geom_array = &self.probe_evaluated_batch.geom_array;
+        let wkbs = geom_array.wkbs();
+        let rects = &geom_array.rects;
+        let distance = &geom_array.distance;
 
         let num_rows = wkbs.len();
 
@@ -575,22 +570,11 @@ impl SpatialJoinBatchIterator {
                     self.join_metrics
                         .join_result_count
                         .add(join_result_metrics.count);
-
-                    // Skip all rects for this probe index since KNN doesn't use them
-                    while self.current_rect_idx < rects.len()
-                        && rects[self.current_rect_idx].0 == self.current_probe_idx
-                    {
-                        self.current_rect_idx += 1;
-                    }
                 }
                 _ => {
                     // Regular spatial join: process all rects for this probe index
-                    while self.current_rect_idx < rects.len()
-                        && rects[self.current_rect_idx].0 == self.current_probe_idx
-                    {
-                        let rect = &rects[self.current_rect_idx].1;
-                        self.current_rect_idx += 1;
-
+                    let rect_opt = &rects[self.current_probe_idx];
+                    if let Some(rect) = rect_opt {
                         let join_result_metrics = self.spatial_index.query(
                             wkb,
                             rect,
@@ -676,7 +660,7 @@ impl SpatialJoinBatchIterator {
         let (build_indices, probe_indices) = match filter {
             Some(filter) => apply_join_filter_to_indices(
                 &partial_build_batch,
-                &self.probe_batch,
+                &self.probe_evaluated_batch.batch,
                 build_indices,
                 probe_indices,
                 filter,
@@ -709,7 +693,7 @@ impl SpatialJoinBatchIterator {
         let result_batch = build_batch_from_indices(
             schema,
             &partial_build_batch,
-            &self.probe_batch,
+            &self.probe_evaluated_batch.batch,
             &build_indices,
             &probe_indices,
             column_indices,
@@ -837,7 +821,6 @@ impl std::fmt::Debug for SpatialJoinBatchIterator {
         f.debug_struct("SpatialJoinBatchIterator")
             .field("max_batch_size", &self.max_batch_size)
             .field("current_probe_idx", &self.current_probe_idx)
-            .field("current_rect_idx", &self.current_rect_idx)
             .field("is_complete", &self.is_complete)
             .field(
                 "build_batch_positions_len",
