@@ -21,7 +21,7 @@ use comfy_table::{Cell, CellAlignment, ColumnConstraint, ContentArrangement, Row
 use datafusion::arrow::util::display::{ArrayFormatter, FormatOptions};
 use datafusion::error::Result;
 use datafusion_common::format::DEFAULT_FORMAT_OPTIONS;
-use datafusion_common::{DataFusionError, ScalarValue};
+use datafusion_common::{config::ConfigOptions, DataFusionError, ScalarValue};
 use datafusion_expr::{ColumnarValue, ReturnFieldArgs, ScalarFunctionArgs, ScalarUDF};
 use sedona_expr::scalar_udf::SedonaScalarUDF;
 use sedona_schema::{datatypes::SedonaType, matchers::ArgMatcher};
@@ -495,6 +495,9 @@ impl DisplayColumn {
                 arg_fields,
                 number_rows: array.len(),
                 return_field,
+                // TODO: Pipe actual ConfigOptions from SedonaContext instead of using defaults
+                // See: https://github.com/apache/sedona-db/issues/248
+                config_options: Arc::new(ConfigOptions::default()),
             };
 
             let format_proxy_value = format_udf.invoke_with_args(args)?;
